@@ -361,6 +361,15 @@ class ExplainerDashboard:
 
         self._store_params(no_param=['explainer', 'tabs', 'server'])
         self._stored_params['tabs'] = self._tabs_to_yaml(tabs)
+        if not hasattr(explainer, "__version__"):
+            raise ValueError(f"The {explainer.__class__.__name__} was generated "
+                    "with a version of explainerdashboard<0.3 and therefore not "
+                    "compatible with this version of ExplainerDashboard due to "
+                    "breaking changes in between major versions! Please rebuild "
+                    f"your {explainer.__class__.__name__} with this version, or "
+                    "downgrade back to explainerdashboard==0.2.20.1!")
+
+        from packaging.version import parse as parse_version
 
         if self.description is None:
             self.description = """This dashboard shows the workings of a fitted
@@ -394,22 +403,31 @@ class ExplainerDashboard:
             else:
                 self.external_stylesheets.append(bootstrap_theme)
 
+        if not hasattr(self.explainer, "onehot_dict"):
+            # explainer generated with explainerdashboard < v0.2.20
+            self.explainer.onehot_dict = self.explainer.cats_dict
+            self.explainer.onehot_cols = self.explainer.cats
+            self.explainer.cat_cols = self.explainer.cats
+            self.explainer.categorical_cols = []
+            self.explainer.categorical_dict = {}
+
         self.app = self._get_dash_app()
 
         if logins is not None:
             if len(logins)==2 and isinstance(logins[0], str) and isinstance(logins[1], str):
-                logins = [logins]
-            assert isinstance(logins, list), \
+                self.logins = [logins]
+                self._stored_params['logins'] = self.logins
+            assert isinstance(self.logins, list), \
                 ("Parameter logins should be a list of lists of str pairs, e.g."
                  " logins=[['user1', 'password1'], ['user2', 'password2']]!")
-            for login in logins:
+            for login in self.logins:
                 assert isinstance(login, list), \
                     ("Parameter logins should be a list of lists of str pairs, "
                      "e.g. logins=[['user1', 'password1'], ['user2', 'password2']]!")
                 assert isinstance(login[0], str) and isinstance(login[1], str), \
                     ("For logins such as [['user1', 'password1']] user1 and "
                      "password1 should be type(str)!")
-            self.auth = dash_auth.BasicAuth(self.app, logins)
+            self.auth = dash_auth.BasicAuth(self.app, self.logins)
         self.app.title = title
 
         assert 'BaseExplainer' in str(explainer.__class__.mro()), \
